/*******************************************************************************
 *                            C++ Verimag
 *    
 *
 *------------------------------------------------------------------------------
 * 	            Distributed and Complex Systems - Gires
 *------------------------------------------------------------------------------
 * executable.h
 *------------------------------------------------------------------------------
 *  Abstract objects for BIP2 Engine
 *  + class Executable  : executable object (port or interaction)
 *------------------------------------------------------------------------------
 * Created by Jacques COMBAZ on Mon Jan 04 2010
 *
 ******************************************************************************/

#ifndef EXECUTABLE_H
#define EXECUTABLE_H

#include "bip-time.h"
#include "clock.h"

#include <iostream>

class Constraint;

/**********************************************
 * Class: Executable
 * Description: object executable by the Engine
 *   can be either an interaction or a port
 *********************************************/
class Executable {
  /*** Protected data members ***/
  protected: 
    unsigned int mMark;                     /* mark for filters */
    Constraint*  mConstraint;               /* time constraint  */
    bool         mDominated;                /* dominated by a priority rule */
                                            /* usefull only when mConstraint == NULL */

  /*** Constructors & Dectructors ***/
  public:
    Executable();
    virtual ~Executable();

  /*** Public Data-Access Methods ***/
  public:
    inline  unsigned int mark()                          const { return mMark; }
    inline  void         setMark(unsigned int m)               { mMark = m; }
    inline  Constraint*  constraint()                    const { return mConstraint; }
    inline  void         setConstraint(Constraint* constraint) { mConstraint = constraint; }
    inline  bool         dominated()                     const { return mDominated; }
    inline  void         setDominated(bool dominated)          { mDominated = dominated; }

  /*** Public Utility Methods ***/
  public:
            void          dominate(Executable&, const BIP::Time&);
    virtual char*         label()                          = 0;
    virtual void          execute(const BIP::Time&)        = 0;
    virtual void          actuate(const BIP::Time&)        = 0;
    virtual BIP::Time     deadline(const BIP::Time&) const = 0;
    virtual BIP::Time     next(const BIP::Time&)     const = 0;
    virtual GenericClock* displayClock()             const = 0;

    friend std::ostream&  operator<<(std::ostream &, Executable&);
    friend std::ostream&  operator<<(std::ostream &, Executable*);
};


#endif

