

#ifndef PORT_H
#define PORT_H

#include <stdio.h>
#ifdef SYMBOLIC
#include  "cudd.h"
#endif

#include  "constraint.h"
#include  "executable.h"

class Executable;
class Atom;
class Connector;


/***************************
 * Class: Port
 * Description: port class
 **************************/
class Port: public Executable {
  /*** Private data members ***/
    char LABEL[1024];

  /*** Protected data members ***/
  protected:
    char*       mName;        /* Name of the port */
    Atom*       mAtomHolder;  /* holder for atomic port */
    Connector*  mConnHolder;  /* holer for conn port */
    bool        mSynced;      /* the holder syncs it     */
    Constraint* mConstraint;  /* the real-time constraint, NULL if no real-time constraint (added by jcombaz) */
    bool        mElected;     /* the scheduler elects it */
    bool        mConnected;   /* i.e. connected, else internal label */
    bool        mInternal;    /* if internal port */
    Port*       mNext;        /* next legal (internal) port */
#ifdef SYMBOLIC
    DdNode*     mBdd;         /* BDD for port */
#endif

  /*** Constructors & Dectructors ***/
  public:
    Port(const char*, 
         const Atom*);
    Port(const char*,
         const Connector*);
    virtual ~Port();

  /*** Public Data-Access Methods ***/
  public:
    inline char*       name()                          const { return mName; }
    inline Atom*       atomHolder()                    const { return mAtomHolder; }
    inline Connector*  connHolder()                    const { return mConnHolder; } 
    inline bool        synced()                        const { return mSynced; }
    inline void        setSynced(bool synced)                { mSynced = synced; }
    inline bool        elected()                       const { return mElected; }
    inline void        setElected(bool elected)              { mElected = elected; }
    inline bool        isConnected()                   const { return mConnected; }
    inline void        setConnected()                        { mConnected = true; }
    inline bool        isInternal()                    const { return mInternal; }
    inline void        setInternal()                         { mInternal = true; }
    inline Port*       next()                          const { return mNext; }
    inline void        setNext(Port* port)                   { mNext = port; }

#ifdef SYMBOLIC

    inline DdNode*     bdd()         const                   { return mBdd; }
    inline void        setBdd(DdNode* bdd)                   { mBdd= bdd; }
#endif

  /*** Public Utility Methods ***/
  public:
    void notifyDependentPorts();
    void execute(const BIP::Time&);
    virtual void actuate(const BIP::Time&);
    void execute();
    BIP::Time   deadline(const BIP::Time&) const;
    BIP::Time   next(const BIP::Time&)     const;
    char*       label();
    GenericClock* displayClock()    const;
    void setLink(Connector*);
#ifdef SYMBOLIC
    void initBdd();
#endif

  /*** Public Methods invoked from appln. ***/
  public:
    bool complete() const;
    bool guard();
};

#endif
