

#ifndef PRIORITY_H
#define PRIORITY_H

#include "clock.h"

class Interaction;
class Compound;
class Executable;

/*******************************
 * Class: Priority
 * Description: 
 *******************************/
class Priority {
  /*** Protected data members ***/
  protected:
    Compound*    mHolder;    /* holder compound  */
    Interaction* mLow;       /* the inhibited    */
    Interaction* mHigh;      /* the inhibitor    */
    bool         mEvaluated; /* true iff the priority rule has been already evaluated */
    bool         mEvaluatedGuard; /* value of the guard of the last evaluation */
    bool         mApplied;   /* true iff the priority rule has been applied */
    BIP::Time    mDelay;     /* delay for the application of priority in real-time constraints */

  /*** Constructors & Dectructors ***/
  public:
    Priority(Compound*, 
	     Interaction*, 
             Interaction*,
             BIP::Time = BIP::Time(0));
    
  virtual ~Priority();

  /*** Public Data-Access Methods ***/
  public:
    inline Compound*    holder()                      const { return mHolder; }
    inline Interaction* low()                         const { return mLow; }
    inline Interaction* high()                        const { return mHigh; }
    inline bool         evaluated()                   const { return mEvaluated; }
    inline void         setEvaluated(const bool evaluated)  { mEvaluated = evaluated; }
    inline bool         evaluatedGuard()                   const { return mEvaluatedGuard; }
    inline void         setEvaluatedGuard(bool guard)            { mEvaluatedGuard = guard; }
    inline bool         applied()                     const { return mApplied; }
    inline void         setApplied(const bool applied)      { mApplied = applied; } 
    inline BIP::Time    delay()                       const { return mDelay; }

  /*** Public Utility Methods ***/
  public:
            void reset();
    virtual void setup();
    virtual void setupPrioLevel();
            void apply(Executable&,  Executable&);

  /*** Public Methods invoked from appln. ***/
  public:
    virtual bool guard() const { return true; }
};

#endif
