#ifndef COMPONENT_H
#define COMPONENT_H

#include <cstdio>

class Compound;
class Scheduler; 

/***********************************
 * Class: Component
 * Description: component base class
 ***********************************/
class Component {

  /*** Protected data members ***/
  protected:
    char*     mName;   /* Name of the component */
    Compound* mHolder; /* Holder component, if any */

  /*** Constructors & Dectructors ***/
  public:
    Component(const char*, 
              Compound*);
    virtual ~Component();


  /*** Public Data-Access Methods ***/
  public:    
    inline char*     name() const   { return mName; }
    inline Compound* holder() const { return mHolder; }

  /*** Public Utility Methods ***/
  public:
    virtual bool isAtom()     const  = 0;
    virtual bool isCompound() const  = 0;
    
    virtual void setup(Scheduler*) = 0;
    virtual void launch() = 0;
    virtual void dump(FILE* file) const = 0;
};

#endif

